#include "my_mesh.h"

#include <iostream>
#include <fstream>


bool check_faces_are_triangles(MyMesh &mesh) {
	for (auto f_it = mesh.faces_begin(); f_it != mesh.faces_end(); ++f_it) {
		size_t n_edges = 0;
		for (auto fe_it = mesh.fe_iter(*f_it); fe_it.is_valid(); ++fe_it) {
			n_edges++;
		}
		if (n_edges != 3) {
			return false;
		}
	}
	return true;
}


bool check_faces_arent_lonely(MyMesh &mesh) {
	for (auto f_it = mesh.faces_begin(); f_it != mesh.faces_end(); ++f_it) {
		auto ff_it = mesh.ff_iter(*f_it);
		if (!ff_it.is_valid()) {
			return false;
		}
	}
	return true;
}


bool check_vertices_arent_lonely(MyMesh &mesh) {
	for (auto v_it = mesh.vertices_begin(); v_it != mesh.vertices_end(); ++v_it) {
		auto ve_it = mesh.ve_iter(*v_it);
		if (!ve_it.is_valid()) {
			return false;
		}
	}
	return true;
}


bool check_edges_arent_lonely(const char *path) {
	using namespace std;
	ifstream f(path);
	string line;
	while (getline(f, line)) {
		istringstream iss(line);
		char first;
		iss >> first;
		if (first == 'l') return false;
	}
	return true;
}


int main(int argc, char *argv[]) {
	using namespace std;

	if (argc != 2) {
		cerr << "Utilisation : " << argv[0] << " [fichier.obj]" << endl;
		return 1;
	}
	MyMesh mesh;
	OpenMesh::IO::read_mesh(mesh, argv[1]);
	cout << "Les faces sont des triangles : "
		 << (check_faces_are_triangles(mesh) ? "oui" : "non")
		 << endl;
	cout << "Les faces ont des voisines : "
		 << (check_faces_arent_lonely(mesh) ? "oui" : "non")
		 << endl;
	cout << "Les sommets sont sur une arête : "
		 << (check_vertices_arent_lonely(mesh) ? "oui" : "non")
		 << endl;
	cout << "Les arêtes sont sur une face : "
		 << (check_edges_arent_lonely(argv[1]) ? "oui" : "non")
		 << endl;
	return 0;
}
